require 'pp'
require 'rexml/document'
require 'json'

class ClaimReceiver

  def make_request_data(xml)
    data,flat = parse_claim(xml)

    # make request skeltion
    ret = {}
    ret["claim_receivev2req"] = claim = {}
    claim["Diagnosis_Information"] = diagnosis_info = {}

    # embed header
    claim["Hospital_Id"]     = flat["/Mml[1]/MmlHeader[1]/mmlCi:CreatorInfo[1]/mmlPsi:PersonalizedInfo[1]/mmlFc:Facility[1]/mmlCm:Id[1]"]
    claim["Patient_ID"]      = flat["/Mml[1]/MmlHeader[1]/masterId[1]/mmlCm:Id[1]"]
    claim["Department_Code"] = flat["/Mml[1]/MmlHeader[1]/mmlCi:CreatorInfo[1]/mmlPsi:PersonalizedInfo[1]/mmlDp:Department[1]/mmlCm:Id[1]"]

    rules = { 
    "Order_Date"    => %r|^/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[\d+\]/content\[1\]/claim:ClaimModule\[1\]/claim:information\[1\]@claim:orderTime|,
    "Perform_Date"  => %r|^/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[\d+\]/content\[1\]/claim:ClaimModule\[1\]/claim:information\[1\]@claim:performTime|,
    }

    flat.each do |k,v|
      rules.each do |kk,r|
        claim[kk] = v if r =~ k
      end
    end

    # diagnosis_info header
    # search mmlbody num
    claim_num = nil
    flat.each do |k,v|
      if %r|^/Mml\[1\]/MmlBody\[1\]/MmlModuleItem\[(\d+)\]/docInfo\[1\]@contentModuleType| =~ k
        claim_num = $1 if v == 'claim'
      end
    end

    rules = {
    "Uid8_Code"      => %r|/MmlModuleItem\[#{claim_num}\]/docInfo\[1\]/docId\[1\]/uid\[1\]$|,
    "Physician_Code" => %r|/MmlModuleItem\[#{claim_num}\]/docInfo\[1\]/mmlCi:CreatorInfo\[1\]/mmlPsi:PersonalizedInfo\[1\]/mmlCm:Id\[1\]$|,
    "InOut"          => %r|/MmlModuleItem\[#{claim_num}\]/content\[1\]/claim:ClaimModule\[1\]/claim:information\[1\]@claim:admitFlag|,
    "Time_Class"     => %r|/MmlModuleItem\[#{claim_num}\]/content\[1\]/claim:ClaimModule\[1\]/claim:information\[1\]@claim:timeClass|,
    "Diagnosis_Uid"  => %r|/MmlModuleItem\[#{claim_num}\]/content\[1\]/claim:ClaimModule\[1\]/claim:information\[1\]@claim:insuranceUid|,
    }
    flat.each do |k,v|
      rules.each do |kk,r|
        diagnosis_info[kk] = v if r =~ k
      end
    end

    #healthinsurance header
    diagnosis_info["HealthInsurance_Information"] = healthinsurance_info = {}
    rules = { 
    "InsuranceProvider_WholeName" => %r|^/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[\d+\]/content\[1\]/mmlHi:HealthInsuranceModule\[1\]/mmlHi:insuranceClass\[1\]$|,
    "InsuranceProvider_Class"     => %r|^/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[\d+\]/content\[1\]/mmlHi:HealthInsuranceModule\[1\]/mmlHi:insuranceClass\[1\]@mmlHi:ClassCode|,
    "InsuranceProvider_Number"    => %r|^/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[\d+\]/content\[1\]/mmlHi:HealthInsuranceModule\[1\]/mmlHi:insuranceNumber\[1\]$|,
    "HealthInsuredPerson_Symbol"  => %r|^/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[\d+\]/content\[1\]/mmlHi:HealthInsuranceModule\[1\]/mmlHi:clientId\[1\]/mmlHi:group\[1\]$|,
    "HealthInsuredPerson_Number"  => %r|^/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[\d+\]/content\[1\]/mmlHi:HealthInsuranceModule\[1\]/mmlHi:clientId\[1\]/mmlHi:number\[1\]$|,
    }
    flat.each do |k,v|
      rules.each do |kk,r|
        healthinsurance_info[kk] = v if r =~ k
      end
    end

    #healthinsurance_info["PublicInsurance_Information"] = publicinsurance_info = Array.new(4,{})
    pi_keys = {}
    flat.keys.each do |k|
      if %r|^/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[\d+\]/content\[1\]/mmlHi:HealthInsuranceModule\[1\]/mmlHi:publicInsurance\[1\]/mmlHi:publicInsuranceItem\[(\d+)\]| =~ k
        pi_keys[$1] ||= []
        pi_keys[$1] << k
      end
    end
    if pi_keys.size > 0
      healthinsurance_info["PublicInsurance_Information"] = publicinsurance_info = []
      rules = {
      "PublicInsurance_Name"       => %r|mmlHi:providerName|,
      "PublicInsurer_Number"       => %r|mmlHi:provider|,
      "PublicInsuredPerson_Number" => %r|mmlHi:recipient|
      }
      pi_keys.values.each_with_index do |k,i|
        publicinsurance_info << {}
        k.each do |kk|
          rules.each do |n,r|
            if r =~ kk
              publicinsurance_info[i][n] = flat[kk]
            end
          end
        end
      end
    end

    # disease_info
    di_keys = {}
    flat.keys.each do |k|
      if %r|/Mml\[1\]/MmlBody\[\d+\]/MmlModuleItem\[(\d+)\]/content\[1\]/mmlRd:RegisteredDiagnosisModule\[1\]| =~ k
        di_keys[$1] ||= []
        di_keys[$1] << k
      end
    end
    if di_keys.size > 0
      diagnosis_info["Disease_Information"] = disease_info = []
      rules = {
      "Disease_Code"      => %r|mmlRd:diagnosis\[1\]@mmlRd:code|,
      "Disease_Name"      => %r|mmlRd:diagnosis\[1\]$|,
      "Disease_StartDate" => %r|mmlRd:startDate|,
      "Disease_EndDate"   => %r|mmlRd:endDate|,
      "Disease_OutCome"   => %r|mmlRd:outcome|,
      "Disease_InOut"     => %r|mmlRd:InOutPatient|
      }
      rules2 = {
      "Disease_Single_Code" => %r|mmlRd:dxItem\[(\d+)\]/mmlRd:name\[1\]@mmlRd:code|,
      "Disease_Single_Name" => %r|mmlRd:dxItem\[(\d+)\]/mmlRd:name\[1\]$|
      }
      rules3 = {
      "Disease_Category"      => %r|mmlRd:category\[(\d+)\]@mmlRd:tableId|,
      "Disease_SuspectedFlag" => %r|mmlRd:category\[(\d+)\]$|
      }
      di_keys.values.each_with_index do |k,i|
        disease_info << {}
        disease_single = {}
        category = {}
        k.each do |kk|
          rules.each do |n,r|
            if r =~ kk
              disease_info[i][n] = flat[kk]
            end
          end
          rules2.each do |n,r|
            if r =~ kk
              disease_single[$1] ||= {}
              disease_single[$1][n] = flat[kk]
            end
          end
          rules3.each do |n,r|
            if r =~ kk
              category[$1] ||= {}
              category[$1][n] = flat[kk]
            end
          end
        end
        disease_info[i]["Disease_Single"] = disease_single.values
        disease_info[i]["Disease_Category_Information"] = category.values
      end
    end

    # Medical_Information
    diagnosis_info["Medical_Information"] = medical_info = []
    flat_claim = flat.select do |k,v|
      %r|/MmlModuleItem\[#{claim_num}\]/| =~ k
    end
    bundle_keys = {}
    flat_claim.each do |k,v|
      if %r|claim:bundle\[(\d+)\]| =~ k
        bundle_keys[$1] ||= []
        bundle_keys[$1] << k
      end
    end
    bundle_keys.values.each_with_index do |k,i|
      medical_info << {}
      medication_info = {}
      rules = {
      "Medical_Class"        => %r|/claim:bundle\[\d+\]@claim:classCode$|,
      "Medical_Class_Codeid" => %r|/claim:bundle\[\d+\]@claim:classCodeId|,
      "Medical_Admin_Codeid" => %r|/claim:bundle\[\d+\]/claim:administration\[1\]@claim:adminCode$|,
      "Medical_Admin_Memo"   => %r|/claim:bundle\[\d+\]/claim:admMemo\[1\]$|,
      "Medical_Class_Number" => %r|/claim:bundle\[\d+\]/claim:bundleNumber\[1\]$|,
      "Medication_Memo_G"    => %r|/claim:bundle\[\d+\]/claim:memo\[1\]$|,
      } 
      rules2 = {
      "Medication_Code"       => %r|claim:item\[(\d+)\]@claim:code|,
      "Medication_Tableid"    => %r|claim:item\[(\d+)\]@claim:tableId|,
      "Medication_Name"       => %r|claim:item\[(\d+)\]/claim:name\[1\]|,
      "Medication_Number"     => %r|claim:item\[(\d+)\]/claim:number\[1\]$|,
      "Medication_Nuber_Code" => %r|claim:item\[(\d+)\]/claim:number\[1\]@claim:numberCode|,
      "Medication_Memo"       => %r|claim:item\[(\d+)\]/claim:memo\[1\]|,
      }
      k.each do |kk|
        rules.each do |n,r|
          medical_info[i][n] = flat_claim[kk] if r =~ kk
        end
        rules2.each do |n,r|
          if r=~ kk
            medication_info[$1] ||= {}
            medication_info[$1][n] = flat_claim[kk]
          end 
        end
      end
      medical_info[i]["Medication_info"] = medication_info.values
    end
    ret
  end

  private

  def parse_elem(elem)
    ret = {:name => elem.expanded_name, :attrs => {}}
    ret[:children] = []
    if elem.has_elements?
      elem.each_element do |c|
        ret[:children] << parse_elem(c)
      end
    else
      if elem.has_text?
        text = elem.texts.join("")
        text.strip!
        ret[:text] = text unless text.empty?
      end
    end
    ret[:attrs] = elem.attributes
    ret
  end
  
  def flatten_data(data,prefix,n,ret)
    key = prefix + "/" + data[:name] + "[#{n}]"
    ret[key] = data[:text] if data[:text] and !data[:text].empty?
    data[:attrs].each do |k,v|
      ret["#{key}@#{k}"] = v if v and !v.empty?
    end
  
    index = {}
    data[:children].each_with_index do |c,i|
      name = c[:name]
      if index[name]
        index[name] += 1
      else
        index[name] = 1
      end
      flatten_data(c,key,index[name],ret)
    end
  end
  
  def parse_claim(xml)
    doc = REXML::Document.new(xml)
    data = parse_elem(doc.root)
    flatten = {}
    flatten_data(data,"",1,flatten)
    [data,flatten]
  end

end

if $0 == __FILE__
  claim = ClaimReceiver.new
  data = claim.make_request_data(File.read(ARGV[0]))
  puts data.to_json
end
